module RouteDataService
  def report_session_route(opts)
    raise 'RouteDataService#report_session_route is not implemented'

  end

  def report_session_route_remove(opts)
    raise 'RouteDataService#report_session_route_remove is not implemented'
  end

end
