#ifndef EXPLOITER_H
#define EXPLOITER_H

VOID ExploiterInit();
VOID ExploiterDoFengShui();
VOID ExploiterSetupFirstChain(ULONGLONG);
VOID ExploiterSetupSecondChain(ULONGLONG, ULONGLONG);
VOID ExploiterRunFirstChain();
VOID ExploiterRunSecondChain();
VOID ExpoiterCleanUp();
ULONGLONG ExploiterGetNtBase();
#endif