#define SCSIZE 2048
unsigned char code[SCSIZE] = "PAYLOAD:";

typedef struct {
	HANDLE hModule;
	HANDLE hMutex;
	HANDLE hProcess;
} EXPLOIT_DATA, *PEXPLOIT_DATA;

#define SIDSTR_SYSTEM _T("s-1-5-18")
#define IsProcessRunningAsSystem(hProc, bResult) IsProcessRunningAsSidString(hProc, SIDSTR_SYSTEM, bResult)
