##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

require 'metasploit/framework/credential_collection'
require 'metasploit/framework/login_scanner/octopusdeploy'

class MetasploitModule < Msf::Auxiliary
  include Msf::Exploit::Remote::HttpClient
  include Msf::Auxiliary::Report
  include Msf::Auxiliary::AuthBrute
  include Msf::Auxiliary::Scanner

  def initialize
    super(
      'Name' => 'Octopus Deploy Login Utility',
      'Description' => %q{
        This module simply attempts to login to an Octopus Deploy server using a specific
        username and password. It has been confirmed to work on version 3.4.4
      },
      'Author' => [ 'James Otten <jamesotten1[at]gmail.com>' ],
      'License' => MSF_LICENSE
    )

    register_options(
      [
        Opt::RPORT(80),
        OptString.new('TARGETURI', [true, 'URI for login. Default is /api/users/login', '/api/users/login'])
      ]
    )
  end

  def run_host(ip)
    cred_collection = build_credential_collection(
      username: datastore['USERNAME'],
      password: datastore['PASSWORD']
    )

    scanner = Metasploit::Framework::LoginScanner::OctopusDeploy.new(
      configure_http_login_scanner(
        cred_details: cred_collection,
        stop_on_success: datastore['STOP_ON_SUCCESS'],
        bruteforce_speed: datastore['BRUTEFORCE_SPEED'],
        connection_timeout: 10,
        http_username: datastore['HttpUsername'],
        http_password: datastore['HttpPassword'],
        uri: datastore['TARGETURI']
      )
    )

    msg = scanner.check_setup
    if msg
      print_error("#{peer} - #{msg}")
      return
    end

    scanner.scan! do |result|
      credential_data = result.to_h
      credential_data.merge!(
        module_fullname: fullname,
        workspace_id: myworkspace_id
      )
      if result.success?
        credential_core = create_credential(credential_data)
        credential_data[:core] = credential_core
        create_credential_login(credential_data)

        print_good "#{Rex::Socket.to_authority(ip, rport)} - Login Successful: #{result.credential}"
      else
        invalidate_login(credential_data)
        vprint_error "#{Rex::Socket.to_authority(ip, rport)} - LOGIN FAILED: #{result.credential} (#{result.status})"
      end
    end
  end
end
