#!/bin/sh

# This script regenerates the LICENSE_GEMS file with the current gem licenses.
# It should be run automatically whenever Metasploit cuts a new release itself.

echo "This file is auto-generated by tools/dev/update_gem_licenses.sh" > LICENSE_GEMS
bundle exec ruby -e "require 'license_finder'; LicenseFinder::CLI::Main.start" | grep , >> LICENSE_GEMS
