// pch.h : precompiled header

#ifndef PCH_H
#define PCH_H

// add precompiled headers here
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#ifndef SECURITY_WIN32
#define SECURITY_WIN32
#endif

#include <Unknwnbase.h>
#include <Sspi.h>
#include <stdio.h>


// Uncomment this line to include debug output
//#define DEBUGTRACE

#ifdef DEBUGTRACE
#define dprintf(...) real_dprintf(__VA_ARGS__)
static void real_dprintf(char* format, ...)
{
	va_list args;
	char buffer[1024];
	va_start(args, format);
	vsnprintf_s(buffer, sizeof(buffer), sizeof(buffer) - 3, format, args);
	strcat_s(buffer, sizeof(buffer), "\r\n");
	OutputDebugStringA(buffer);
}
#else
#define dprintf(...)
#endif


//#include "ReflectiveLoader.h"
#include "LocalNegotiator.h"
#include "Services/service.h"
#include "Services/elevatorService.h"
#include "base64.h"
#include "spnegotokenhandler/spnego.h"
#include "spnegotokenhandler/derparse.h"
#include "spnegotokenhandler/spnegoparse.h"
#include "RogueWinRM.h"

#endif //PCH_H
