#include <windows.h>
#include <ntverp.h>

#define VER_FILETYPE                VFT_DLL
#define VER_FILESUBTYPE             VFT2_UNKNOWN
#define VER_FILEDESCRIPTION_STR     "Byakugan"

#define VER_INTERNALNAME_STR        "byakugan.DLL"
#define VER_ORIGINALFILENAME_STR    "byakugan.DLL"

#include "common.ver"
