module LoginDataService

  def logins(opts)
    raise 'LoginDataService#logins is not implemented'
  end

  def create_credential_login(opts)
    raise 'LoginDataService#create_credential_login is not implemented'
  end

  def update_login(hosts)
    raise 'LoginDataService#update_login is not implemented'
  end
end