# -*- coding: binary -*-
module Msf::Exploit::Remote::HTTP::NagiosXi::URIs

  # Returns the Nagios XI Login URL
  #
  # @return [String] Nagios XI Login URL
  def nagios_xi_login_url
    normalize_uri(target_uri.path, 'login.php')
  end

  # Returns the Nagios XI Backend URL
  #
  # @return [String] Nagios XI Backend URL
  def nagios_xi_backend_url
    normalize_uri(target_uri.path, 'index.php')
  end

  # Returns the Nagios XI Installation URL
  #
  # @return [String] Nagios XI Install URL
  def nagios_xi_install_url
    normalize_uri(target_uri.path, 'install.php')
  end

end
