RSpec.shared_examples_for 'Msf::DBManager::ExploitedHost' do

  if ENV['REMOTE_DB']
    before {skip("Not used for remote data service")}
  end

  it { is_expected.to respond_to :each_exploited_host }
  it { is_expected.to respond_to :exploited_hosts }
end