// swift-tools-version:5.1
// The swift-tools-version declares the minimum version of Swift required to build this package.

import PackageDescription

let package = Package(
    name: "example",
    products: [
        .executable(name: "Test", targets: ["Test"]),
    ],
    dependencies: [
	   .package(url: "https://github.com/ViacomInc/URLSessionDecodable.git", .exact("0.1.0"))
    ],
    targets: [
        // Targets are the basic building blocks of a package. A target can define a module or a test suite.
        // Targets can depend on other targets in this package, and on products in packages which this package depends on.
        .target(
            name: "Test",
            dependencies: ["URLSessionDecodable"]),
    ]
)
