<header role="banner">
  <h1>Unknown action</h1>
</header>
<main role="main" id="container">
  <%= render "rescues/message_and_suggestions", exception: @exception, exception_wrapper: @exception_wrapper %>
</main>
