# -*- coding: binary -*-
module Rex
module Post
module Meterpreter
module Extensions
module Kiwi

TLV_TYPE_KIWI_CMD        = TLV_META_TYPE_STRING | (TLV_EXTENSIONS + 100)
TLV_TYPE_KIWI_CMD_RESULT = TLV_META_TYPE_STRING | (TLV_EXTENSIONS + 101)

end
end
end
end
end
