module ExploitDataService

  def report_exploit_attempt(host, opts)
    raise 'ExploitDataService#report_exploit_attempt is not implemented'
  end

  def report_exploit_failure(opts)
    raise 'ExploitDataService#report_exploit_failure is not implemented'
  end

  def report_exploit_success(opts)
    raise 'ExploitDataService#report_exploit_success is not implemented'
  end

end