# -*- coding: binary -*-
# NAT-PMP protocol support
#
# @author Jon Hart <jhart@spoofed.org>

module Rex::Proto::NATPMP
  include Rex::Proto::NATPMP::Constants
  include Rex::Proto::NATPMP::Packet
end
