#pragma once

#include <wincrypt.h>
#pragma comment(lib, "crypt32.lib")

char* base64_encode(const unsigned char*, unsigned short, unsigned short*);
unsigned char* base64_decode(const char*, unsigned short, unsigned short*);

