defmodule Awesomer do
  @moduledoc """
  Documentation for Awesomer.
  """

  @doc """
  Hello world.

  ## Examples

      iex> Awesomer.hello
      :world

  """
  def hello do
    :world
  end
end
