module CredentialDataService

  def creds(opts)
    raise 'CredentialDataService#creds is not implemented'
  end

  def create_credential(opts)
    raise 'CredentialDataService#create_credential is not implemented'
  end

  def update_credential(opts)
    raise 'CredentialDataService#update_credential is not implemented'
  end

  def delete_credential(opts)
    raise 'CredentialDataService#delete_credential is not implemented'
  end
end