package vuln;

/*
 * This is just a placeholder class.  Link implements this so it can be
 * replaced after compilation with Map.Entry since a normal compiler will not
 * allow creating a class that implements an interface through inheritance.
 *
 * See Sami Koivu's original blog post for details.
 */
public interface Test {
	
	public Object getValue() throws Exception;

}
