import javax.sound.midi.*;

public class MyController implements ControllerEventListener {
     
  public MyController() {
  	
  }
  
  public void controlChange(ShortMessage event) {
  }

}  // close inner class