;      Title:  Win32 Network Shell
;  Platforms:  Windows NT 4.0, Windows 2000, Windows XP, Windows 2003
;     Author:  hdm[at]metasploit.com



[BITS 32]

%include "win32_stage_boot_bind.asm"
%include "win32_stage_shell.asm"
